@@ -114,10 +114,8 @@ public class Main {
             decoder.setDecodeSources(ApkDecoder.DECODE_SOURCES_NONE);
         }
         if (cli.hasOption("d") || cli.hasOption("debug")) {
-            decoder.setDebugMode(true);
-        }
-        if (cli.hasOption("debug-line-prefix")) {
-            decoder.setDebugLinePrefix(cli.getOptionValue("debug-line-prefix"));
+            System.err.println("SmaliDebugging has been removed in 2.1.0 onward. Please see: https://github.com/iBotPeaches/Apktool/issues/1061");
+            System.exit(1);
         }
         if (cli.hasOption("b") || cli.hasOption("no-debug-info")) {
             decoder.setBaksmaliDebugMode(false);
@@ -151,7 +149,7 @@ public class Main {
             // make out folder manually using name of apk
             String outName = apkName;
             outName = outName.endsWith(".apk") ? outName.substring(0,
-                    outName.length() - 4) : outName + ".out";
+                    outName.length() - 4).trim() : outName + ".out";
 
             // make file from path
             outName = new File(outName).getName();
@@ -201,6 +199,7 @@ public class Main {
             apkOptions.forceBuildAll = true;
         }
         if (cli.hasOption("d") || cli.hasOption("debug")) {
+            System.out.println("SmaliDebugging has been removed in 2.1.0 onward. Please see: https://github.com/iBotPeaches/Apktool/issues/1061");
             apkOptions.debugMode = true;
         }
         if (cli.hasOption("v") || cli.hasOption("verbose")) {
@@ -273,19 +272,13 @@ public class Main {
                 .create("r");
 
         Option debugDecOption = OptionBuilder.withLongOpt("debug")
-                .withDescription("Decode in debug mode. Check project page for more info.")
+                .withDescription("REMOVED (DOES NOT WORK): Decode in debug mode.")
                 .create("d");
 
         Option analysisOption = OptionBuilder.withLongOpt("match-original")
                 .withDescription("Keeps files to closest to original as possible. Prevents rebuild.")
                 .create("m");
 
-        Option debugLinePrefix = OptionBuilder.withLongOpt("debug-line-prefix")
-                .withDescription("Smali line prefix when decoding in debug mode. Default is \"a=0;// \".")
-                .hasArg(true)
-                .withArgName("prefix")
-                .create();
-
         Option apiLevelOption = OptionBuilder.withLongOpt("api")
                 .withDescription("The numeric api-level of the file to generate, e.g. 14 for ICS.")
                 .hasArg(true)
@@ -293,7 +286,7 @@ public class Main {
                 .create();
 
         Option debugBuiOption = OptionBuilder.withLongOpt("debug")
-                .withDescription("Builds in debug mode. Check project page for more info.")
+                .withDescription("Sets android:debuggable to \"true\" in the APK's compiled manifest")
                 .create("d");
 
         Option noDbgOption = OptionBuilder.withLongOpt("no-debug-info")
@@ -369,7 +362,6 @@ public class Main {
 
         // check for advance mode
         if (isAdvanceMode()) {
-            DecodeOptions.addOption(debugLinePrefix);
             DecodeOptions.addOption(debugDecOption);
             DecodeOptions.addOption(noDbgOption);
             DecodeOptions.addOption(keepResOption);
@@ -416,7 +408,6 @@ public class Main {
             allOptions.addOption((Option)op);
         }
         allOptions.addOption(analysisOption);
-        allOptions.addOption(debugLinePrefix);
         allOptions.addOption(debugDecOption);
         allOptions.addOption(noDbgOption);
         allOptions.addOption(keepResOption);
@@ -470,7 +461,7 @@ public class Main {
         // print out more information
         System.out.println(
                 "For additional info, see: http://ibotpeaches.github.io/Apktool/ \n"
-                        + "For smali/baksmali info, see: http://code.google.com/p/smali/");
+                        + "For smali/baksmali info, see: https://github.com/JesusFreke/smali");
     }
 
     private static void setupLogging(Verbosity verbosity) {
